/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id:$ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */


import koala.util.*;
import koala.gui.*;
import koala.look.*;
import koala.kit.*;


import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Point;
import java.util.Vector;
import java.awt.Event;

public class DrawingPanel extends Control implements ToolConsumer, Look, Slider {
    private Vector looks = new Vector();
    private Vector locations = new Vector();

    public DrawingPanel() {
	setLook(this);
	setBehavior(new DragBehavior(this));
    }
    
    //============================================================
    //      implements: Look
    //============================================================

    
    public synchronized void draw(DrawingContext dc) {
	for(int i = 0; i < looks.size(); i++) {
	    Point p = (Point)locations.elementAt(i);
	    dc.move(p.x, p.y);
	    ((Look)looks.elementAt(i)).draw(dc);
	}
    }
    Dimension size;
    public synchronized Dimension request(DrawingContext dc) {
	return size = new Dimension(400, 400);
    }
    public Dimension size(DrawingContext dc) {
	return (size != null) ? size : request(dc);
    }
    //============================================================
    //      implements: Slider
    //============================================================

    private int ox, oy, lastx, lasty;
    DrawingContext idc;
    DrawingTool tool;
    public void select(int x, int y, Event evt) {
	ox = x;
	oy = y;
	lastx = x;
	lasty = y;
	idc = new DrawingContext(this, getGraphics(), 
				new Rectangle(size()), getStyle());
	if(tool != null) tool.drawFeedback(idc, ox, oy, lastx, lasty);
    }
    
    
    public void drag(int dx, int dy, Event evt) {
	if(tool != null) tool.eraseFeedback(idc, ox, oy, lastx, lasty);
	lastx = evt.x;
	lasty = evt.y;
	if(tool != null) tool.drawFeedback(idc, ox, oy, lastx, lasty);
    }
    
    
    public void drop(int x, int y, Event evt) {
	if(tool != null) {
	    tool.eraseFeedback(idc, ox, oy, lastx, lasty);
	    tool.newInstance(this, ox, oy, lastx, lasty);
	    repaint();
	}
    }
    
    public void jumpto(int x, int y, Event evt) {
	// nothing
    }
    
    //============================================================
    //      implements: ToolConsumer
    //============================================================

    public void setTool(DrawingTool t) {
	tool = t;
    }
    
    public DrawingTool getTool() {
	return tool;
    }

    public  synchronized void add(int x, int y, Look l) {
	looks.addElement(l);
	locations.addElement(new Point(x, y));
    }
}
