/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Combinator.java,v 1.4 1996/06/05 15:34:47 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import java.awt.Font;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.Vector;


/**
 * A Combinator is an abstract used as base class for looks that handle
 * children. It contains a list of children and can add/remove elements from
 * this list. A combinator does not define the 'draw' and request methods
 * required by the Look interface.
 */
abstract public class Combinator implements Look {
    protected static final int GAP = 4;
    protected static final int MARGIN = 4;

    protected Vector children;
    protected int gap = GAP;
    protected int margin = MARGIN;
    protected Dimension size;

   /**
    * Creates a new empty Combinator.
    */    
    public Combinator() {
	children = new Vector();
    }

   /**
    * Add a Look to this combinator.
    */    
    public Combinator add(Look item) {
	children.addElement(item);
	return this;
    }

   /**
    * Remove a Look from this combinator.
    */    
    public Combinator remove(Look item) {
	children.removeElement(item);
	return this;
    }

   /**
    * returns the previously computed size of the Combinator. This size should
    * have been computed and stored by the last 'request' call.
    */
    public Dimension size(DrawingContext dc) {
	return (size != null) ? size : request(dc);
    }
}
