/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Composite.java,v 1.5 1996/06/18 08:31:29 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import java.awt.Event;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Rectangle;


/**
 * A base class for Composite (or Container) objects.
 *
 * A Composite is a Widget that contains children (i.e other Widgets). Just
 * like Widgets, Composites have a Style, a Look and a Behavior. A Common Look
 * for Composites is BorderLook.
 *
 * A Composite delegates its events to its Behavior and delegates the paint and
 * update requests to its Look.
 */
public class Composite extends java.awt.Panel implements Widget {
    
    protected Behavior behavior = null;
    protected Look look = null;
    protected Style sty = null;
    
    /*  --- This code should be multiply inherited  --- start --- */
    public void setBehavior(Behavior mgr) {behavior = mgr; };
    public Behavior getBehavior() { return behavior; };
    public void appendBehavior(Behavior b) {
	WidgetCode.appendBehavior(this, b);
    }
    public void prependBehavior(Behavior b) {
	WidgetCode.prependBehavior(this, b);
    }
    public void removeBehavior(Behavior b) {
	WidgetCode.removeBehavior(this, b);
    }
    
    public void setLook(Look mgr) {look = mgr; };
    public Look getLook() { return look; };
    public void appendLook(Look l) {
	WidgetCode.appendLook(this, l);
    }
    public void prependLook(Look l) {
	WidgetCode.prependLook(this, l);
    }
    public void removeLook(Look l) {
	WidgetCode.removeLook(this, l);
    }
    
    public void setStyle(Style s) {sty = s; setBackground(sty.background);};
    public Style getStyle() { return sty; };
    
    public void paint(java.awt.Graphics g) {
	WidgetCode.paint(this, this, g);
    }
    
    public void update(java.awt.Graphics g) {
	WidgetCode.update(this, this, g, true);
    }

    public final boolean postEvent(Event e) {
	return Toolkit.dispatchEvent(this, e);
    }

    public boolean handleEvent(Event evt) {
	return WidgetCode.handleEvent(this, evt);
    }
    /*  --- This code should be multiply inherited  --- end --- */
    
    public Composite() {
	setLayout(null);
	setStyle(Style.getDefaultStyle());
    }
    
    public Insets insets() {
 	int i =  getStyle().shadowThickness;
 	return new Insets(i, i, i, i);
    }

    public Rectangle clipper() {
	Dimension size = size();
	Insets i = super.insets();
	return new Rectangle(i.left, i.top, size.width-i.left-i.right,
			     size.height-i.top-i.bottom);
    }
}
