/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Control.java,v 1.5 1996/06/03 16:34:30 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import koala.util.App;

import java.awt.Event;
import java.awt.Dimension;
import java.awt.Rectangle;

/**
 * A base class for Control (or Primitive) objects.
 *
 * A Control is a Widget that *does not* contains children (i.e it is a leaf in
 * the widget tree). A Control has a Style, a Look and a Behavior.
 *
 * A Control delegates its events to its Behavior and delegates the paint and
 * update requests to its Look.
 */
public class Control extends java.awt.Canvas implements Widget {
    
    Behavior behaviorMgr = null;
    protected Look lookMgr = null;
    Style sty = Style.getDefaultStyle();
    
    /*  --- This code should be multiply inherited  --- start --- */
    public void setBehavior(Behavior mgr) {behaviorMgr = mgr; };
    public Behavior getBehavior() { return behaviorMgr; };
    public void appendBehavior(Behavior b) {
	WidgetCode.appendBehavior(this, b);
    }
    public void prependBehavior(Behavior b) {
	WidgetCode.prependBehavior(this, b);
    }
    public void removeBehavior(Behavior b) {
	WidgetCode.removeBehavior(this, b);
    }
    
    public void setLook(Look mgr) {lookMgr = mgr; };
    public Look getLook() { return lookMgr; };
    public void appendLook(Look l) {
	WidgetCode.appendLook(this, l);
    }
    public void prependLook(Look l) {
	WidgetCode.prependLook(this, l);
    }
    public void removeLook(Look l) {
	WidgetCode.removeLook(this, l);
    }
    
    public void setStyle(Style s) {sty = s; setBackground(sty.background);};
    public Style getStyle() { return sty; };
    
    public void paint(java.awt.Graphics g) {
	WidgetCode.paint(this, this, g);
    }
    
    public void update(java.awt.Graphics g) {
	WidgetCode.update(this, this, g, true);
    }

    public boolean postEvent(Event e) {
	App.assert(this == e.target);
	return Toolkit.dispatchEvent(this, e);
    }
    
    public boolean handleEvent(Event evt) {
 	return WidgetCode.handleEvent(this, evt);
    }
    /*  --- This code should be multiply inherited  --- end --- */
    
    public Dimension minimumSize() {
	if(lookMgr != null) {
	    return lookMgr.request(new DrawingContext(this, getGraphics(), 
						      new Rectangle(size()), 
					getStyle()));
	}
	else return new Dimension(50, 20);
    }
    
    public Dimension preferredSize() {
	return minimumSize();
    }

    public Rectangle clipper() {
	return new Rectangle(size());
    }
}
