/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Designation.java,v 1.1 1996/06/05 15:34:48 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import java.awt.Event;

/** 
 * This interface must be implemented by tools that intend to serve as
 * selectors.
 * 
 * A Selector can be a single widget, like a CheckButton or a Composite widget
 * like a RadioBox.
 */
public interface Designation {

   /**
    * <bold>ACTION</bold>: invoked when a new choice is selected.
    * 
    * @param w The widget that becomes the current selection.
    * @param evt The AWT Event that triggered the action.
    */    
    public void designate(Widget w, Event evt);

   /**
    * Return the current selection.
    */
    public Object current();

   /**
    * Add a Command that is triggered everytime the selection changes.
    */    
    public Designation onDesignate(Command c);


   /**
    * Remove a Command from the Command lis.
    */    
    public Designation offDesignate(Command c);
}
