/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Dialog.java,v 1.3 1996/06/03 16:34:31 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import koala.look.BorderLook;
import java.awt.Event;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Insets;

/**
 * A base class for Dialogs.
 *
 * A Dialog is a toplevel Widget using its own Window. Unlike Frames, it can be
 * modal.
 *
 * It is easy to consider that a Dialog is a subclass of Window, though this
 * inheritance is implemented as copy-paste.
 *
 * A Dialog delegates its events to its Behavior and delegates the paint and
 * update requests to its Look.  */
public class Dialog extends java.awt.Dialog implements Widget {

    static protected Style style = new Style("Dialog");
    Behavior behavior = null;
    protected Look look = null;
    protected Style sty = null;
    
    /*  --- This code should be multiply inherited  --- start --- */
    public void setBehavior(Behavior mgr) {behavior = mgr; };
    public Behavior getBehavior() { return behavior; };
    public void appendBehavior(Behavior b) {
	WidgetCode.appendBehavior(this, b);
    }
    public void prependBehavior(Behavior b) {
	WidgetCode.prependBehavior(this, b);
    }
    public void removeBehavior(Behavior b) {
	WidgetCode.removeBehavior(this, b);
    }
    
    public void setLook(Look mgr) {look = mgr; };
    public Look getLook() { return look; };
    public void appendLook(Look l) {
	WidgetCode.appendLook(this, l);
    }
    public void prependLook(Look l) {
	WidgetCode.prependLook(this, l);
    }
    public void removeLook(Look l) {
	WidgetCode.removeLook(this, l);
    }
    
    public void setStyle(Style s) {sty = s; setBackground(sty.background);};
    public Style getStyle() { return sty; };
    
    public void paint(java.awt.Graphics g) {
	WidgetCode.update(this, this, g, false);
    }
    
    public void update(java.awt.Graphics g) {
	WidgetCode.update(this, this, g, true);
    }

    public boolean postEvent(Event e) {
	return Toolkit.dispatchEvent(this, e);
    }
    
    public boolean handleEvent(Event evt) {
 	return WidgetCode.handleEvent(this, evt);
    }
    /*  --- This code should be multiply inherited  --- start --- */

    public Dialog(Frame parent, String title, boolean modal) {
	super(parent, title, modal);
	setStyle(style);
	setLook(new BorderLook());
    }
    
    public Insets insets() {
	Insets r = super.insets();
	int i =  getStyle().shadowThickness;
	return new Insets(r.top+i, r.left+i, r.bottom+i, r.right+i);
    }

    public Rectangle clipper() {
	Dimension size = size();
	Insets i = super.insets();
	return new Rectangle(i.left, i.top, size.width-i.left-i.right,
			     size.height-i.top-i.bottom);
    }
}
