/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: FocusManager.java,v 1.7 1996/05/30 07:40:08 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import koala.look.Icon;
import koala.util.Symbol;
import java.awt.Event;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Color;
import java.util.Observable;


/**
 * A sample implementation for a Focus Look - unused
 */
class FocusLook implements Look {
    static final int GAP = 1;
    static final int SIZE = 5;
    public void draw(DrawingContext dc) {
	int x[] = new int[3];
	int y[] = new int[3];
	Dimension s = dc.size();
	dc.setColor(Color.black);
	x[0] = GAP; x[1] = GAP+SIZE; x[2] = GAP;
	y[0] = GAP; y[1] = GAP; y[2] = GAP+SIZE;
	dc.fillPolygon(x, y, 3);
	dc.setColor(dc.style().foreground);
    }
    public Dimension request(DrawingContext dc) {
	return new Dimension(0, 0);
    }
    public Dimension size(DrawingContext dc) {
	return new Dimension(0, 0);
    }
}

public class FocusManager extends Observable implements EventFilter {
    static protected final int CLICK_TO_TYPE = 1;
    static protected final int FOLLOW_MOUSE = 2;
    static Symbol POLICIES[] = {
	new Symbol("clicktotype", CLICK_TO_TYPE),
	new Symbol("followmouse", FOLLOW_MOUSE)
    };
    protected final int policy = 
    Symbol.getSymbol(Gui.FOCUS_POLICY_P, POLICIES, CLICK_TO_TYPE);

    private final boolean VARYLOOK = false;

    Component focus = null;
    Look look = null;
    public boolean handleEvent(Component c, Event evt) {
	switch(policy) {
	case CLICK_TO_TYPE:
	    if(evt.id == Event.MOUSE_DOWN) {
		setFocus(c);
	    }
 	    if(evt.id == Event.KEY_PRESS || (evt.id == Event.KEY_RELEASE)) {
 		if(focus != null) {
		    if (focus != c) {
			// specify we want to redirect this event
			evt.target = focus;
			if(focus.postEvent(evt)) {
			    return true;
			}
		    }
		    if((evt.id == Event.KEY_PRESS) && (evt.key == 0x09)) {
			navigate(c, evt);
			return true;
		    }
 		}
 	    }
	    break;
	case FOLLOW_MOUSE:
	    if(evt.id == Event.MOUSE_ENTER) {
		setFocus(c);
		//make this works with native AWT
		c.requestFocus();
	    }
	    break;
	}
	return false;
    }

    public void setFocus(Component c) {
	if(focus == c) return;
	if(VARYLOOK && (look == null)) {
	    look = new Icon(c, "focus.gif");
	}
	if(focus != null) {
	    Widget wfocus = (Widget)focus;
	    if(VARYLOOK) {
		wfocus.removeLook(look);
		focus.repaint();
	    }
	}
	if(VARYLOOK) {
	    Widget w = (Widget)c;
	    w.appendLook(look);
	    c.repaint();
	}
	focus = c;
	setChanged();
	notifyObservers();
    }

    void navigate(Component c, Event e) {
	switch(e.key) {
	case Event.UP:		// bug: we should to get those keys, here.
	case Event.LEFT:
	    previous();
	    break;
	case 0x09:		// TAB
	case Event.DOWN:
	case Event.RIGHT:
	    next();
	    break;
	}
    }

    public void next() {
	Container parent = focus.getParent();
	Component child;
	if(parent != null) {
	    int n = parent.countComponents();
	    for(int i = 0; i < n; i++) {
		child = parent.getComponent(i);
		if(child == focus) {
		    while(true) {
			// find the first enabled 'next'
			int nexti = (i < n-1) ? i+1 : 0;
			Component next = parent.getComponent(nexti);
			if(next.isEnabled()) {
			    setFocus(next);
			    return;
			}
			i = nexti;
		    }
		}
	    }
	}
    }

    public void previous() {
	Container parent = focus.getParent();
	Component child;
	if(parent != null) {
	    int n = parent.countComponents();
	    for(int i = 0; i < n; i++) {
		child = parent.getComponent(i);
		if(child == focus) {
		    while(true) {
			int previ = (i == 0) ? n-1 : i-1;
			Component prev = parent.getComponent(previ);
			if(prev.isEnabled()) {
			    setFocus(prev);
			    return;
			}
			i = previ;
		    }
		}
	    }
	}
    }
}
