/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Relief.java,v 1.3 1996/05/30 06:31:59 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import koala.util.Symbol;

import java.awt.Color;
import java.awt.Graphics;

public class Relief {
    
    static protected double bcoef = 0.85; // used to compute brighter colors
    static protected double dcoef = 0.5;  // used to compute darker colors
    
    public static final int FLAT = 0;
    public static final int SUNKEN = 1;
    public static final int RAISED = 2;
    public static final int GROOVE = 3;
    public static final int RIDGE = 4;
    
    public static final int THICKNESS = 2;

    static Symbol RELIEFS[] = {
	new Symbol("flat", FLAT),
	new Symbol("sunken", SUNKEN),
	new Symbol("raised", RAISED),
	new Symbol("groove", GROOVE),
	new Symbol("ridge", RIDGE)
    };
    
    /**
     * Returns a brighter version of the color.
     */
    static public Color brighter(Color c) {
	return new Color(Math.min((int)(c.getRed()  *(1/bcoef)), 255), 
			 Math.min((int)(c.getGreen()*(1/bcoef)), 255),
			 Math.min((int)(c.getBlue() *(1/bcoef)), 255));
    }

    /**
     * Returns a darker version of the color.
     */
    static public Color darker(Color c) {
	return new Color(Math.max((int)((double)c.getRed()  * dcoef), 0), 
			 Math.max((int)((double)c.getGreen()* dcoef), 0),
			 Math.max((int)((double)c.getBlue() * dcoef), 0));
    }

    /**
     * Returns a darker version of the color.
     */
    static public Color halfDarker(Color c) {
	return new Color(Math.max((int)((double)c.getRed()  * dcoef*1.5), 0), 
			 Math.max((int)((double)c.getGreen()* dcoef*1.5), 0),
			 Math.max((int)((double)c.getBlue() * dcoef*1.5), 0));
    }

    /**
     * Returns a grayed out version of the color.
     */
    static public Color grayed(Color c) {
	return Color.gray;
    }


   /**
    * Draws a 3D rectangle, as requested by size, type and thickness.
    */    
    static public void drawRect(Graphics g, int x, int y, int w, int h,
			    int relief, int thickness) {
	boolean raised;
	if(relief == FLAT) return;
	// GROOVE and RIDGE are achieved with a succession of RAISED/SUNKEN
	if((relief == GROOVE) || ( relief == SUNKEN)) {
	    raised = false;
	}
	else {
	    raised = true;
	}
	for(int i = 0; i < thickness; i++) {
	    {
 		Color c = g.getColor();
 		Color brighter = brighter(c);
 		Color darker = darker(c);
		
 		g.setColor(raised ? brighter : darker);
 		g.drawLine(x, y, x, y + h);
 		g.drawLine(x + 1, y, x + w - 1, y);
 		g.setColor(raised ? darker : brighter);
 		g.drawLine(x + 1, y + h, x + w, y + h);
 		g.drawLine(x + w, y, x + w, y + h - 1);
 		g.setColor(c);

		x += 1; 
		y += 1; 
		w -= 2; 
		h -= 2;
		if((relief == GROOVE) || (relief == RIDGE)) {
		    raised = ! raised;
		}
	    }
	}
    }


   /**
    * Draws a 3D rectangle, as requested by size, type and thickness.
    */    
    static public void drawOval(Graphics g, int x, int y, int w, int h,
			       int relief, int thickness) {
	boolean raised;
	if(relief == FLAT) return;
	// GROOVE and RIDGE are achieved with a succession of RAISED/SUNKEN
	if((relief == GROOVE) || ( relief == SUNKEN)) {
	    raised = false;
	}
	else {
	    raised = true;
	}
	for(int i = 0; i < thickness; i++) {
	    {
 		Color c = g.getColor();
 		Color brighter = brighter(c);
 		Color darker = darker(c);
		
 		g.setColor(raised ? brighter : darker);
 		g.drawArc(x, y, w, h, 45, 180);
 		g.setColor(raised ? darker : brighter);
 		g.drawArc(x, y, w, h, 225, 180);
 		g.setColor(c);

		x += 1; 
		y += 1; 
		w -= 2; 
		h -= 2;
		if((relief == GROOVE) || (relief == RIDGE)) {
		    raised = ! raised;
		}
	    }
	}
    }
    
    public static int getRelief(String name, int def) {
	return Symbol.getSymbol(name, RELIEFS, def);
    }
    
    public static String toString(int r) {
	if((r < FLAT) || (r > RIDGE)) {
	    throw new IllegalArgumentException("invalid relief");
	}
	return RELIEFS[r].name;
    }
}
