/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Selection.java,v 1.1 1996/06/18 13:44:45 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import java.awt.Event;
import java.util.Enumeration;


/**
 * Defines the base interface for objects able to perform a selection.
 */
public interface Selection {

   /**
    * Constant defining single selection mode.
    */    
    static public final int SINGLE = 1;

   /**
    * Constant defining multiple selection mode.
    */    
    static public final int MULTIPLE = 2;

   /**
    * Sets selection policy. In SINGLE selection mode, selecting an item
    * deselects any previously selected item.
    */
    public void setPolicy(int policy);

   /**
    * Selects the specified item. If selection policy is SINGLE, deselects any
    * other selected item.
    */
    public void select(Object item);

   /**
    * Selects all the items. Selection policy must be MULTIPLE.
    */    
    public void selectAll();

   /**
    * Unselects the specified item.
    */    
    public void unselect(Object item);

   /**
    * Unselects any selected item.
    */    
    public void unselectAll(Object item);

   /**
    * @return the current selection.
    */    
    public Enumeration selection();

   /**
    * Adds the specified command to the 'select' command list. The select
    * command list is triggered everytime a new item is selected.
    */
    public void onSelect(Command c);

   /**
    * Removes the specified command from the 'select' command list. The select
    * command list is triggered everytime a new item is selected.
    */
    public void offSelect(Command c);

   /**
    * Adds the specified command to the 'modified' command list. The select
    * command list is triggered everytime the selection changes.
    */
    public void onModified(Command c);

   /**
    * Removes the specified command from the 'modified' command list. The
    * select command list is triggered everytime the selection changes.
    */
    public void offModified(Command c);
    
}
