/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Style.java,v 1.4 1996/06/18 08:31:29 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import koala.util.*;

import java.awt.Font;
import java.awt.Color;
import java.awt.Graphics;

public class Style {

    static public final int NORMAL = 1;
    static public final int BOLD = 2;
    static public final int ITALIC = 3;
    static protected Style normal = null;
    static protected Style bold = null;
    static protected Style italic = null;
    
    static {
	// This file should contain default values for styles
	App.loadProfile(".gui");
    }

    static public final Font defaultFont = 
            Font.getFont(Gui.FONT_P, new Font("Helvetica", Font.PLAIN, 12));
    static public final int defaultThickness = 
            Integer.getInteger(Gui.SHADOW_THICKNESS_P, 1).intValue();
    static public final Color defaultForeground = 
            Color.getColor(Gui.FOREGROUND_P, Color.black);
    static public final Color defaultBackground =
            Color.getColor(Gui.BACKGROUND_P, Color.gray);
    static public final int defaultRelief =
            Relief.getRelief(Gui.RELIEF_P, Relief.RAISED);
    
    
    public Font font;
    public Color foreground;
    public Color background;
    public int shadowThickness;
    public int relief;

    
    static Style getDefaultStyle() {
	return getDefault(NORMAL);
    }

    static Style getDefault(int s) {
	switch(s) {
	case BOLD:
	    if(bold == null) {
		bold = new  Style("bold");
	    }
	    return bold;
	case ITALIC:
	    if(italic == null) {
		italic = new  Style("italic");
	    }
	    return italic;
	default:
	    if(normal == null) {
		normal = new  Style("default");
	    }
	    return normal;
	}
    }
    
    public Style(Font f, Color fg, Color bg, int t) {
	init(f, fg, bg, t);
    }
    
    public Style(String name) {
	font = Font.getFont(name + ".font", defaultFont);
	foreground = Color.getColor(name + ".foreground", 
				    defaultForeground);
	background = Color.getColor(name + ".background", 
				    defaultBackground);
	shadowThickness = 
            Integer.getInteger(name + ".shadowThickness",
			       defaultThickness).intValue();
	relief = Relief.getRelief(name + ".relief", defaultRelief);
    }
    
    protected void init(Font f, Color fg, Color bg, int t) {
	font = f;
	foreground = fg;
	background = bg;
	shadowThickness = t;
    }
    
    public String toString() {
	return "Style[font=" + font.toString() + "," +
	              "foreground=" + foreground.toString() + "," +
	              "background=" + background.toString() + "," +
	              "shadowThickness=" + shadowThickness + "," +
	              "relief=" + Relief.toString(relief) + "]";
    }
    
    public void apply(Graphics g) {
	g.setFont(font);
	g.setColor(foreground);
    }
}
