/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Toolkit.java,v 1.8 1996/06/18 08:31:30 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import java.awt.Event;
import java.awt.Component;
import java.awt.Container;
import java.awt.Point;
import java.awt.Insets;
import java.util.Vector;


class EventFilterList extends Vector implements EventFilter {
    
    public boolean handleEvent(Component c, Event evt) {
	boolean r = false;
	int n = size();
	for(int i = 0; i < n; i++) {
	    EventFilter f = (EventFilter) elementAt(i);
	    r |= f.handleEvent(c, evt);
	}
	return r;
    }
}

class DebugFilter implements EventFilter {
    public boolean handleEvent(Component c, Event e) {
	System.out.println("handling " + c);
	return true;
    }
}


public class Toolkit {
    static EventFilterList filters = null;
    static EventFilterList fallbacks = null;
    static {
	//	prependFilter(new DebugFilter()); 
	appendFilter(new FocusManager());
    }

    static boolean filterEvent(Component c, Event e) {
	return ((filters != null) && (filters.handleEvent(c, e)));
    }

    static boolean fallbackEvent(Component c, Event e) {
	return ((fallbacks != null) && (fallbacks.handleEvent(c, e)));
    }

    static void prependFilter(EventFilter f) {
	if(filters == null) {
	    filters = new EventFilterList();
	}
	filters.insertElementAt(f, 0);
    }

    static void appendFilter(EventFilter f) {
	if(filters == null) {
	    filters = new EventFilterList();
	}
	filters.addElement(f);
    }

    static boolean dispatchEvent(Component c, Event e) {
	Widget w = (Widget)c;

	if(filterEvent(c, e)) {
	    return true;
	}
	
	if (c.handleEvent(e)) {
	    return true;
	}
	// if this event was not previously handled, dispatch it to parent
 	for(Component parent = c.getParent(); parent != null;
	    parent = parent.getParent()) {
 	    //	    e.translate(x, y);
 	    if (parent.handleEvent(e)) {
 		return true;
 	    }
 	}

	if(fallbackEvent(c, e)) {
	    return true;
	}
	
	return false;
    }
    
    static GrabManager grabber = null;
    public static GrabManager getGrabManager() {
	if(grabber == null) {
	    grabber =  new GrabManager();
	    prependFilter(grabber);
	}
	return grabber;
    }

    static public void addGrab(Popup w) {
	getGrabManager().addGrab(w);
    }
    
    static public void removeGrab(Popup w) {
	getGrabManager().removeGrab(w);
    }

    static public java.awt.Window getWindow(Component c) {
	Container r = null;
	for(Container p = c.getParent(); p != null; p = p.getParent()) {
	    r = p;
	}
	return (java.awt.Window)r;
    }

    static public void rootCoords(Point coords, Component c) {
	for(Container p = c.getParent(); p != null; p = p.getParent()) {
	    coords.translate(p.location().x, p.location().y);
	    Insets insets = p.insets();
	    coords.translate(-insets.left, -insets.top);
	}
    }
}
