/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Widget.java,v 1.14 1996/06/05 15:34:51 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */


package koala.gui;

import koala.util.*;

import java.awt.Rectangle;
import java.awt.Component;
import java.awt.Container;
import java.awt.Insets;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.Dimension;
import java.util.Vector;

/** 
 * Defines the interface for classes that are widgets.
 *
 * @version	$Revision: 1.14 $
 */
public interface Widget {
    
    
    /**
     * Sets the behavior of the widget. 
     * @param mgr becomes the new behavior on that widget
     */
    public void setBehavior(Behavior b);
    public void appendBehavior(Behavior b);
    public void prependBehavior(Behavior b);
    public void removeBehavior(Behavior b);
    
    /**
     * Returns the current behavior on the widget. 
     */
    public Behavior getBehavior();
    
    /**
     * Sets the look of the widget. 
     * @param look becomes the new look on that widget
     */
    public void setLook(Look look);
    public void appendLook(Look look);
    public void prependLook(Look look);
    public void removeLook(Look look);
    
    /**
     * Returns the current look of the widget. 
     */
    public Look getLook();
    
    /**
     * Sets the style of the widget. 
     * @param sty becomes the new style on that widget
     */
    public void setStyle(Style sty);
    
    /**
     * Returns the current style of the widget. 
     */
    public Style getStyle();

   /**
    * Returns the 'visible part' of the widget. The clipper is used to compute
    * the region where the Look will draw.
    */
    public Rectangle clipper();
}

