/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Window.java,v 1.3 1996/06/24 09:06:17 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import java.awt.Event;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Point;

public class Window extends java.awt.Window implements Widget {
    
    static private boolean dbuf = true;
    protected Behavior behavior = null;
    protected Look look = null;
    protected Style sty = null;
    
    /*  --- This code should be multiply inherited  --- start --- */
    public void setBehavior(Behavior mgr) {behavior = mgr; };
    public Behavior getBehavior() { return behavior; };
    public void appendBehavior(Behavior b) {
	WidgetCode.appendBehavior(this, b);
    }
    public void prependBehavior(Behavior b) {
	WidgetCode.prependBehavior(this, b);
    }
    public void removeBehavior(Behavior b) {
	WidgetCode.removeBehavior(this, b);
    }
    
    public void setLook(Look mgr) {look = mgr; };
    public Look getLook() { return look; };
    public void appendLook(Look l) {
	WidgetCode.appendLook(this, l);
    }
    public void prependLook(Look l) {
	WidgetCode.prependLook(this, l);
    }
    public void removeLook(Look l) {
	WidgetCode.removeLook(this, l);
    }
    
    public void setStyle(Style s) {sty = s; setBackground(sty.background);};
    public Style getStyle() { return sty; };
    
    public void paint(java.awt.Graphics g) {
	WidgetCode.paint(this, this, g);
    }
    
    public void update(java.awt.Graphics g) {
	WidgetCode.update(this, this, g, true);
    }
    
    

    // WARNING: This code is intended to correct a bug in AWT read FRAME-BUG
    // for more details I see no 100% perfect workaround in Java: this one has
    // a bug when you resize a window from the top: it moves the window.

    // Here is the original code:
    /*
    public boolean postEvent(Event e) {
	return  Toolkit.dispatchEvent(this, e);
    }

    public boolean handleEvent(Event evt) {
	return WidgetCode.handleEvent(this, evt);
    }
    */
    
    //   --  WORKAROUND - start - jml - 5/29/96.
    private boolean resize = false;
    private boolean moved = false;
    private int rootx = 0, rooty = 0;

    public void layout() {
	super.layout();
	resize = true;
    }
    
    public boolean postEvent(Event e) {
	if(e.id == Event.WINDOW_MOVED) {
	    if(resize) {
		if(moved) {
		    Point pos = location();
		    if((pos.x != rootx) || (pos.y != rooty)) {
			move(rootx, rooty);
		    }
		}
	    }
	    else {
		rootx = e.x;
		rooty = e.y;
		moved = true;
	    }
	    resize = false;
	}
	//   --  WORKAROUND - end - jml - 5/29/96.
	return  Toolkit.dispatchEvent(this, e);
    }
    
    public boolean handleEvent(Event evt) {
 	return WidgetCode.handleEvent(this, evt);
    }
    /*  --- This code should be multiply inherited  --- end --- */

    /*  --- This code should be multiply inherited  --- end --- */
    
    public Window(Frame parent) {
	super(parent);
	setLayout(null);
	setStyle(Style.getDefaultStyle());
    }
    
    public Insets insets() {
	int i =  getStyle().shadowThickness;
	return new Insets(i, i, i, i);
    }

    public Rectangle clipper() {
	Dimension size = size();
	Insets i = super.insets();
	return new Rectangle(i.left, i.top, size.width-i.left-i.right,
			     size.height-i.top-i.bottom);
    }
}
