/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: ActivateBehavior.java,v 1.1 1996/06/18 08:31:30 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import java.awt.Component;
import java.awt.Container;
import java.awt.Event;

/**
 * ActiveBehavoir is used to detect double clicks and <return> keystrokes.
 */
public class ActivateBehavior implements Behavior {
    protected Activation target;

   /**
    * Builds a new ActivateBehavior, with specified target
    *
    * @param t Behavior's target.
    */    
    public ActivateBehavior(Activation t) {
	target = t;
    }

   /**
    * Handles double clicks and <return>. calls the activate method of the
    * target.
    * @return true if the event is caught, false otherwise.
    */
    public boolean handleEvent(Component c, Event evt) {
	switch (evt.id) {
	case Event.MOUSE_DOWN: {
	    switch(evt.clickCount) {
 	    case 2:
 		target.activate((Widget)evt.target, evt);
 		return true;
	    default:
		return false;
	    }
	}
 	case Event.KEY_PRESS: 
 	    if(evt.key != Gui.ENTER_KEY) break;
 	    target.activate((Widget)evt.target, evt);
 	    return true;
 	}
        return false;
    }
}
