/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Button.java,v 1.9 1996/06/12 15:39:27 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.util.*;
import koala.gui.*;
import koala.look.*;

import java.awt.Event;
/**
 * The Button widget class.
 * 
 * @version	1.0 - jml - 3/20/96.
 */
public class Button extends Control implements ButtonWidget {
    protected boolean pressed;
    protected Command activate;
    protected String label;
    static Style style = new Style("Button");
    
    public Button() {
	initButton("", null, new ButtonLook(this));
    }

   /**
    * Builds a new Button widget, with the specified look.
    */
    public Button(Look l) {
	initButton("", null, new ButtonLook(this, l));
    }

   /**
    * Builds a new Button widget with the specified Look. The command argument
    * is added to the activate command list.
    */
    public Button(Look l, Command c) {
	initButton("", c, new ButtonLook(this, l));
    }

    /**
     * Creates a new button widget, that will trigger the given Command.
     * @param label the label displayed on the button
     * @param cmd the command triggered when the button is activated
     */
    public Button(String l, Command c) {
	initButton(l, c, new ButtonLook(this));
    }

    /**
     * Creates a new button widget, that will trigger the given Command.
     * @param label the label displayed on the button
     * @param cmd the command triggered when the button is activated
     */
    public Button(String l) {
	initButton(l, null, new ButtonLook(this));
    }
    
    private void initButton(String s, Command c, Look l) {
	label = s;
	setBehavior(new ButtonBehavior(this));
	setLook(l);
	pressed = false;
	setStyle(style);
	if(c != null) {
	    onActivate(c);
	}
    }
    
    //============================================================
    //      implements: LabelWidget
    //============================================================

   /**
    * Sets the specified String as label. The Button is not redrawn.
    */    
    public void setLabel(String s) {
	label = s;
    }

   /**
    * @return The current label.
    */    
    public String getLabel() {
	return label;
    }
    
    //============================================================
    //      implements: Press
    //============================================================

   /**
    * @return true if the Button is pressed, false otherwise.
    */    
     public boolean isPressed() {
 	return pressed;
     }

   /**
    * Sets pressed state to 'true' and update button's appearance.
    */    
    public void press(Widget what, Event e) {
	pressed = true;
	repaint();
    }
    

   /**
    * Sets pressed state to 'false' and update button's appearance.
    */    
    public void release(Widget what, Event e) {
	pressed = false;
	repaint();
    }
    
    //============================================================
    //      implements: Activation
    //============================================================

   /**
    * Triggers commands for the 'activate' command list.
    */    
    public void activate(Widget what, Event e) {
	WidgetCode.execute(activate, this, null);
    }

   /**
    * Appends the specified command to the 'activate' command list.
    */
    public void onActivate(Command cmd) {
	activate = WidgetCode.onCommand(activate, cmd);
    }

   /**
    * Removes the specified command from the 'activate' command list.
    */
    public void offActivate(Command cmd) {
	activate = WidgetCode.offCommand(activate, cmd);
    }

    public String toString() {
	return getClass().getName() + "[" + label + "]";
    }
}
