/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: ButtonBehavior.java,v 1.5 1996/06/05 15:34:52 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import java.awt.Component;
import java.awt.Event;

/**
 * Describes the common behavior of a Button.
 */
public class ButtonBehavior implements Behavior {
    private ButtonWidget target;
    // 'pressed' may have any semantic for the target, so we have to duplicate
    // the state here, to hold a physical 'pressed state'
    private boolean pressed = false; 

   /**
    * Builds a new ButtonBehavior objects, using the ButtonWidget as target.
    */    
    public ButtonBehavior(ButtonWidget bc) {
	target = bc;
    }

   /**
    * Handles events, calling actions on the 'target' object.
    *
    * @return true is the event is caught, false otherwise.
    */
    public boolean handleEvent(Component c, Event evt) {
	switch (evt.id) {
	case Event.KEY_PRESS:
	    if(evt.key != Gui.ENTER_KEY) break;
	    
	case Event.MOUSE_DOWN:
	    target.press(target, evt);
	    pressed = true;
	    return true;
	    
	case Event.MOUSE_ENTER:
	    if(pressed)
		target.press(target, evt);
	    return true;

	case Event.MOUSE_EXIT:
	    if(pressed) {
		target.release(target, evt);
		pressed = false;
	    }
	    return true;

	case Event.KEY_RELEASE:
	    if(evt.key != Gui.ENTER_KEY) break;
	case Event.MOUSE_UP:
	    if(pressed) {
		target.release(target, evt);
		target.activate(target, evt);
	    }
	    pressed = false;
	    return true;
	}
        return false;
    }
}
