/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: ButtonGroup.java,v 1.6 1996/06/05 15:34:53 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */


package koala.kit;

import koala.gui.*;
import java.util.Vector;
import java.util.Observable;
import java.awt.Component;
import java.awt.Event;


/**
  * A ButtonGroup is intended to make a choice between different ButtonWidget
  * that can be contained in a Composite.
  */
public class ButtonGroup extends Observable implements Designation {
    protected Vector elements;
    ButtonWidget selected = null;

   /**
    * Builds a new empty ButtonGroup.
    */    
    public ButtonGroup() {
	elements = new Vector(0, 1);
    }

   /**
    * Builds a new ButtonGroup containing ButtonWidget.
    */    
    public void add(ButtonWidget bc) {
	elements.addElement(bc);
    }

   /**
    * Removes the specified ButtonWidget from the group.
    */    
    public void remove(ButtonWidget bc) {
	elements.removeElement(bc);
    }

   /**
    * Implements the Designation Task.
    *
    * call the press() method for the ButtonWidget being designated. calls
    * release() for the previously selected item.
    */
    public synchronized void designate(Widget target, Event evt) {
	if(selected == target) return;
	for(int i = 0; i < elements.size(); i++) {
	    // we cannot add children that are not ButtonWidget
	    ButtonWidget child = (ButtonWidget)elements.elementAt(i);
	    if(target == child) {
		child.press(child, evt);
		child.activate(child, evt);
		selected = child;
		setChanged();
		notifyObservers();
	    }
	    else if(child.isPressed()) {
		child.release(child, evt);
		//		child.activate(child, evt);
	    }
	}
    }
    
   /**
    * Return the current selection.
    */
    public Object current() {
	return selected;
    }

   /**
    * Add a Command that is triggered everytime the selection changes.
    */    
    public Designation onDesignate(Command c) {
	return this;
    }

   /**
    * Remove a Command from the Command lis.
    */    
    public Designation offDesignate(Command c) {
	return this;
    }
}
