/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: CheckButton.java,v 1.5 1996/06/05 15:34:54 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.util.*;
import koala.gui.*;
import koala.look.*;
import java.util.*;
import java.awt.Event;

/*
 * The CheckButton widget class.
 * 
 * @version	1.0 - jml - 3/20/96.
 */

public class CheckButton extends Button implements Observer {
    protected BooleanModel variable = null;
    static Style style = new Style("CheckButton");
    
    /**
     * Creates a new CheckButton widget, that will trigger the given Command.
     * @param label the label displayed on the button
     * @param cmd the command triggered when the button is activated
     */
    public CheckButton(String l, Command c) {
	initCheckButton(l, c, (BooleanModel)new ActiveBoolean());
    }
    
    /**
     * Creates a new CheckButton widget, that will update the given variable.
     * @param label the label displayed on the button
     * @param v the model controlled by this view
     */
    public CheckButton(String l, BooleanModel v) {
	initCheckButton(l, null, v);
    }
    
    private void initCheckButton(String l, Command c, BooleanModel v) {
	setLook(new HBox().add(new CheckLook(this)).add(new LabelLook(this)));
	label = l;
	variable = v;
	v.addObserver(this);
	setStyle(style);
    }
    

    /**
     * Sets pressed state to 'true' and update button's appearance.
     */  
     public void press(Widget what, Event e) {
	 pressed = !pressed;
	 repaint();
     }
    
    /**
     * Sets pressed state to 'false' and update button's appearance.
     */   
     public void release(Widget what, Event e) {
	 pressed = !pressed;
 	 repaint();
     }

   /**
    * Updates button state according to variable's value.
    */ 
    public void update(Observable o, Object arg) {
	pressed = variable.get();
	repaint();
    }
    
   /**
    * Modifies variable's value and triggers commands for the 'activate'
    * command list.
    */
    public void activate(Widget what, Event e) {
	variable.set(! variable.get());
	pressed = variable.get();
	WidgetCode.execute(activate, this, e);
    }
}
