/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: ComboBox.java,v 1.2 1996/06/18 13:44:45 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import koala.look.*;
import koala.util.*;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.Event;
import java.awt.Point;
import java.awt.Dimension;
import java.util.Enumeration;


class ComboUpdateCmd implements Command {
    ComboBox cb;
    Listbox lb;
    ComboUpdateCmd(ComboBox c, Listbox l) {
	cb = c;
	lb = l;
    }
    
    public void execute(Component target, Event evt) {
	Enumeration e = lb.selection.elements();
	cb.tf.setLabel((String)e.nextElement());
    }
}


class ComboPopup extends Popup {
    Listbox list;
    public ComboPopup(Frame f, ComboBox cb) {
	super(f);
	setLayout(new BorderLayout());
	list = new Listbox();
	list.onActivate(new HideCmd(this));
	list.onDesignate(new ComboUpdateCmd(cb, list));
	add("Center", new ScrolledWindow(list ,list));
	for(int i = 0; i < 300; i++) {
	    list.insert("item " + i);
	}
    }

    //============================================================
    //      implements: Popup
    //============================================================

    public void gotGrab(Popup previous) {
	// empty
    }
    public void lostGrab(Popup next) {
	// empty
    }
}


/**
 * The ComboBox widget class.
 *
 */
public class ComboBox extends Composite implements Command {
    static private Style combostyle = new Style("ComboBox");
    static final String ICONFILE = App.getStringProp("ComboBox.icon",
						     "arrowdown.gif");

    TextField tf;
    Button arrow;
    Menu menu;
    Popup popup;

   /**
    * Builds a new ComboBox widget.
    */    
    public ComboBox(Frame f) {
	setStyle(combostyle);
	setLayout(new BorderLayout());
	add("Center", tf = new TextField());
	add("East", arrow =
	    new Button(new VCPlug(new Icon(this, ICONFILE)), this));
	
	popup = new ComboPopup(f, this);
    }
    
    //============================================================
    //      implements: Command
    //============================================================

    public void execute(Component target, Event evt) {
	Point p = location();
	Dimension d = size();
	Toolkit.rootCoords(p, this);

	popup.reshape(p.x, p.y+d.height, d.width, 200);
	popup.pack();

	popup.reshape(p.x, p.y+d.height, d.width, popup.size().height);
	popup.show();
    }
}
