/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: FileSelector.java,v 1.5 1996/05/30 07:40:15 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import koala.util.*;
import koala.look.*;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Event;
import java.util.Enumeration;
import java.io.*;

interface FileSelection {
    public FileSelection setPath(String path);
    public FileSelection setFilter(String path);
    public String selection();
}


public class FileSelector extends Composite implements FileSelection,
    FilenameFilter, Command {
    protected String path;
    protected String filter;
    protected Listbox files;
    protected Listbox dirs;

    public FileSelector(String p, String f) {
	Listbox l;
	path = p;
	filter = f;
	files = new Listbox();
	files.activate = this;
	fillIt();
	setLayout(new BorderLayout());
	add("Center", new ScrolledWindow(files, files));
    }

    public void fillIt() {
	Look dir = new VCPlug(new Icon(this, "dir.gif"));
	Look file = new VCPlug(new Icon(this, "file.gif"));
	HBox box;
	File f = new File(path), child;
	String []contents = f.list(this);
	files.insert("..", new HBox(4, 0)
		     .add(new VCPlug(new Icon(this, "updir.gif")))
		     .add(new VString("upper directory")));
	for(int i = 0; i < contents.length; i++) {
	    files.insert(contents[i], box = new HBox(4, 0));
	    child = new File(path, contents[i]);
	    box.add(child.isDirectory() ? dir : file);
	    box.add(new VString(contents[i]));
	}
	files.commit();
    }
    
    public FileSelection setPath(String p) {
	files.removeAll();
	path = p;
	fillIt();
	return this;
    }
    
    public FileSelection setFilter(String f) {
	filter = f;
	return this;
    }
    
    public boolean accept(File  dir, String  name) {
	return true;
    }
	
    public void execute(Component target, Event evt) {
	File newpath = new File(path, selection());
	if(newpath.isDirectory()) {
	    setPath(newpath.getAbsolutePath());
	}
	
    }

    public String selection() {
	Enumeration e = files.selection.elements();
	return (String)e.nextElement();
    }
}
