/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: HorizontalLayout.java,v 1.4 1996/06/03 16:34:36 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import java.awt.*;

/**
 * A simple layout that arranges its components horizontally
 * all the components are given the same height and keep their own width.
 * 
 * @author 	  - jml - 1/19/96.
 */

public class HorizontalLayout implements LayoutManager {

    final static private int HGAP = 
      Integer.getInteger("HorizontalLayout.hgap", 0).intValue();
    final static private int HMARGIN = 
      Integer.getInteger("HorizontalLayout.hmargin", 0).intValue();
    final static private int VMARGIN = 
      Integer.getInteger("HorizontalLayout.vmargin", 0).intValue();

    private int hgap;		// space inserted between children
    private int hmargin;	// margin added to each child
    private int vmargin;	// vertical margin for the Composite
    /**
     * Constructs a new HorizontalLayout.
     */
    public HorizontalLayout() {
	this(HGAP, HMARGIN, VMARGIN);
    }
    
    /**
     * Constructs a new HorizontalLayout with specific gap and margin.
     */
    public HorizontalLayout(int gap, int h, int v) {
	hgap = gap;
	hmargin = h;
	vmargin = v;
    }

    /**
     * Adds the specified named component to the layout.
     * @param name the String name
     * @param comp the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {}

    /**
     * Removes the specified component from the layout.
     * @param comp the component to be removed
     */
    public void removeLayoutComponent(Component comp) {}

    
    public Dimension minimumLayoutSize(Container target) {
	return preferredLayoutSize(target);
    }
     
    public Dimension preferredLayoutSize(Container target) {
	Dimension dim = new Dimension(0, 0);
	int width = 0;
	int height = 0;
	Component children[] = target.getComponents();
	
	for(int i=0; i < children.length; i++) {
	    height = Math.max(height, children[i].preferredSize().height);
	    width += children[i].preferredSize().width;
	}
	width += hgap * (children.length + 1) + hmargin*2*children.length;
	Insets insets = target.insets();
	dim.width = width + insets.left + insets.right;
	dim.height = height + insets.top + insets.bottom + vmargin*2;

	return dim;
    }
    
    
    /**
     * Lays out the specified container.
     * @param target the component being laid out
     * @see Container
     */
    public void layoutContainer(Container target) {
	Insets insets = target.insets();
	int top = insets.top + vmargin, left = insets.left;
	int height = target.size().height - top - insets.bottom - vmargin;
	Component children[] = target.getComponents();
	
	left += hgap;
	for(int i=0; i < children.length; i++) {
	    int w = children[i].preferredSize().width + hmargin*2;
	    children[i].reshape(left, top, w, height);
	    left += w;
	}
    }
     
    /**
     * Returns the String representation of this BorderLayout's values.
     */
    public String toString() {
	return getClass().getName() + ",hgap=" + hgap + "]";
    }

}
