/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id:$ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */


package koala.kit;

import koala.util.GeometryManager;
import koala.util.GeometryComputer;
import koala.util.GeometryRenderer;

import java.awt.*;

/**
 * A Generic layout that uses the GeometryComputer.
 * 
 * @author 	   - jml - 6/11/96.
 */

public class LayoutComputer implements LayoutManager {

    private int hgap = 0;		// space inserted between children
    private int hmargin = 0;	// margin added to each child
    private int vmargin = 0;	// vertical margin for the Composite
    /**
     * Adds the specified named component to the layout.
     * @param name the String name
     * @param comp the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {}

    /**
     * Removes the specified component from the layout.
     * @param comp the component to be removed
     */
    public void removeLayoutComponent(Component comp) {}

    
    public Dimension minimumLayoutSize(Container target) {
	return preferredLayoutSize(target);
    }

    void applyScheme(GeometryManager gm) {
	gm.expandWidth(20);
	gm.expandHeight(2);
	gm.distributeHorizontally();
	gm.flushTop(10);
	gm.flushBottom(10);
    }
     
    public Dimension preferredLayoutSize(Container target) {
 	Insets insets = target.insets();
	Component children[] = target.getComponents();
	Rectangle boxes[] = new Rectangle[children.length];
	Dimension d;
	for(int i=0; i < children.length; i++) {
	    d = children[i].preferredSize();
	    boxes[i] = new Rectangle(0, 0, d.width, d.height);
	}
	GeometryComputer gc = new GeometryComputer(boxes);
	applyScheme(gc);
	Dimension res = gc.size();
	res.width += insets.left + insets.right;
	res.height +=  insets.top + insets.bottom;
	System.out.println(res);
	return res;
    }

    /**
     * Lays out the specified container.
     * @param target the component being laid out
     * @see Container
     */
    public void layoutContainer(Container target) {
	Insets insets = target.insets();
	int top = insets.top + vmargin, left = insets.left;
	int height = target.size().height - top - insets.bottom - vmargin;
	int width = target.size().width - left - insets.right - hmargin;
	Component children[] = target.getComponents();
	Rectangle boxes[] = new Rectangle[children.length];

	for(int i=0; i < children.length; i++) {
	    Dimension s = children[i].preferredSize();
	    boxes[i] = new Rectangle(0, 0, s.width, s.height);
	}
	
	GeometryRenderer gr = new GeometryRenderer(boxes, width, height);
	applyScheme(gr);

	for(int i=0; i < children.length; i++) {
	    children[i].reshape(boxes[i].x+left, boxes[i].y+top,
				boxes[i].width, boxes[i].height);
	}
    }
}
