/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Menu.java,v 1.6 1996/06/12 15:39:28 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import koala.look.*;
import koala.util.*;
import java.awt.Component;
import java.awt.Event;
import java.awt.Point;
import java.awt.Insets;
import java.awt.Container;

public class Menu extends Popup implements MenuContainer, Slider {
    
    static protected Style style = new Style("Menu");
    static protected Style substyle = new Style("MenuItem");

    public Menu(Frame f) {
	super(f);
	setBehavior(new DragBehavior(this));
	setLayout(new VerticalLayout(0, 0));
	setLook(new BorderLook());
	setStyle(style);
    }

    private void insert(ButtonWidget c) {
	c.setStyle(substyle);
	c.setBehavior(null);
    }

    void translate(Point coords, Component c) {
	for(Container p = c.getParent(); p != null; p = p.getParent()) {
	    coords.translate(p.location().x, p.location().y);
	    Insets insets = p.insets();
	    coords.translate(-insets.left, -insets.top);
	}
    }
    
    MenuContainer postedFrom;
    public void postMenu(ButtonWidget from, Menu what) {
	what.postedFrom = this;
	Component fromc = (Component)from;
	Point coords = new Point(fromc.size().width, fromc.location().y);
	//	translate(coords, fromc);
	coords.translate(location().x, location().y);

	what.reshape(coords.x, coords.y, 1, 1);
	what.validate();
	what.pack();
	what.show();
	
    }
    public void unpostMenu(ButtonWidget from, Menu what) {
	what.hide();
    }

    //============================================================
    //      implements: MenuContainer
    //============================================================

    public void close() {
	hide();
	postedFrom.close();
    }
    
    public MenuContainer addItem(ButtonWidget c) {
	insert(c);
	super.add((Component)c);
	return this;
    }
    
    public MenuContainer addItem(String layout, ButtonWidget c) {
	insert(c);
	super.add((Component)c);
	return this;
    }
    
    public MenuContainer removeItem(ButtonWidget c) {
	super.remove((Component)c);
	return this;
    }
    
    public MenuContainer addMenu(Menu m, String title) {
	CascadeButton c = new CascadeButton(title, m); 
	c.setLook(new ButtonLook(c, new HBox().add(new VString(title))
		  .add(new RPlug(new Icon(this, "arrowright.gif")))));
	insert(c);
	super.add((Component)c);
	return this;
    }
    
    public MenuContainer addMenu(String layout, Menu m, String title) {
	CascadeButton c = new CascadeButton(title, m);
	insert(c);
	super.add(layout, (Component)c);
	return this;
    }
    
    //============================================================
    //      implements: Slider
    //============================================================


    synchronized Component locateChild(int x, int y) {
	for(int i = 0; i <countComponents(); i++) {
	    Component c = getComponent(i);
	    if(c.bounds().inside(x, y)) {
		return c.isEnabled() ? c : this;
	    }
	}
	return this;
    }
    
    ButtonWidget holder = null;
    public void select(int x, int y, Event e) {
	Component to;
	to = locateChild(e.x, e.y);
	if(to instanceof ButtonWidget) {
	    ButtonWidget target = (ButtonWidget)to;
	    target.press(target, e);
	    holder = target;
	}
    }
    
    public void drag(int dx, int dy, Event e) {
	Component to;
	to = locateChild(e.x, e.y);

	if((to != null) && (to instanceof ButtonWidget) && (to != holder)) {
	    if(holder != null) holder.release(holder, e);
	    ButtonWidget target = (ButtonWidget)to;
	    target.press(target, e);
	    holder = target;
	}
    }
    
    public void drop(int x, int y, Event e) {
	Component to;
	to = locateChild(e.x, e.y);
	ButtonWidget target = (ButtonWidget)to;
	//	if(to != holder) {
	holder.release(holder, e);
	target.activate(target, e);
	holder = null;
	    //	}
	close();
    }

    public void jumpto(int x, int y, Event e) {
	// nothing
    }
    
    public void gotGrab(Popup previous) {
	
    }
    public void lostGrab(Popup next) {
	if((holder != null) && !(holder instanceof CascadeButton)) {
	    holder.release(holder, null);
	    holder = null;
	}
    }
}
