/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: MenuBar.java,v 1.6 1996/06/12 15:39:28 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import koala.look.*;
import java.awt.Component;
import java.awt.Container;
import java.awt.Event;
import java.awt.Point;
import java.awt.Insets;


public class MenuBar extends Composite implements MenuContainer, Slider {
    static final boolean KEEP_OPEN = false;
    static protected Style style = new Style("MenuBar");
    static protected Style substyle = new Style("MenuItem");
    static final protected int HMARGIN = 5;
    static final protected int VMARGIN = 2;
    ButtonGroup group;
    
    public MenuBar() {
	group = new ButtonGroup();
	setBehavior(new DragBehavior(this));
	setLayout(new HorizontalLayout(0, HMARGIN, VMARGIN));
	setLook(new BorderLook());
	setStyle(style);
    }
    
    public MenuBar(ButtonGroup sg) {
	group = sg;
    }

    private void insert(ButtonWidget c) {
	c.setStyle(substyle);
	c.setBehavior(null);
	group.add(c);	
    }
    
    //============================================================
    //      implements: MenuContainer
    //============================================================

    public MenuContainer addMenu(Menu m, String title) {
	CascadeButton c = new CascadeButton(title, m); 
	insert(c);
	super.add((Component)c);
	return this;
    }
    
    public MenuContainer addMenu(String layout, Menu m, String title) {
	CascadeButton c = new CascadeButton(title, m); 
	insert(c);
	super.add(layout, (Component)c);
	return this;
    }

    public MenuContainer addItem(ButtonWidget c) {
	insert(c);
	super.add((Component)c);
	return this;
    }
    
    public MenuContainer addItem(String layout, ButtonWidget c) {
	insert(c);
	super.add(layout, (Component)c);
	return this;
    }
    
    public MenuContainer removeItem(ButtonWidget c) {
	super.remove((Component)c);
	group.remove(c);
	return this;
    }

    void translate(Point coords, Component c) {
	for(Container p = c.getParent(); p != null; p = p.getParent()) {
	    coords.translate(p.location().x, p.location().y);
	    Insets insets = p.insets();
	    //	    coords.translate(insets.left, insets.top);
	}
    }
    
    public void postMenu(ButtonWidget from, Menu what) {
	what.postedFrom = this;
	Component fromc = (Component)from;
	Point coords = new Point(fromc.location().x, size().height);
	translate(coords, fromc.getParent());

	what.reshape(coords.x, coords.y, 1, 1);
	what.validate();
	what.pack();
	what.show();
    }
    
    public void unpostMenu(ButtonWidget from, Menu what) {
	//	what.invalidate();
	what.hide();
    }
    
    //============================================================
    //      implements: Slider
    //============================================================

    ButtonWidget holder = null;
    public void select(int x, int y, Event e) {
	//	Toolkit.addGrab((java.awt.Window)this.getParent());
	Component to;
 	Point loc =  ((Component)e.target).location();
 	e.translate(loc.x, loc.y);
	to = locate(e.x, e.y);
	if(to instanceof ButtonWidget) {
	    // else we clicked on the menubar
	    ButtonWidget target = (ButtonWidget)e.target;
	    target.press(target, e);
	    holder = target;
	}
    }
    
    public void drag(int dx, int dy, Event e) {
	Component to;
 	Point loc =  ((Component)e.target).location();
 	e.translate(loc.x, loc.y);
	to = locate(e.x, e.y);
	if((to != null) && (to instanceof ButtonWidget) && (to != holder)) {
	    holder.release(holder, e);
	    ButtonWidget target = (ButtonWidget)to;
	    target.press(target, e);
	    holder = target;
	}
    }
    
    public void drop(int x, int y, Event e) {
	Component to;
 	Point loc =  ((Component)e.target).location();
 	e.translate(loc.x, loc.y);
	to = locate(e.x, e.y);
	if(KEEP_OPEN) {
	    if(to != holder) {
		holder.release(holder, e);
	    }
	}
	else {
	    holder.release(holder, e);
	}
    }

    public void jumpto(int x, int y, Event e) {
	// nothing
    }

    public void enter(int x, int y, Event evt) {
	// nothing
    }

    public void leave(int x, int y, Event evt) {
	// nothing
    }

    public void close() {
	// nothing
    }
}
