/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Scrollbar.java,v 1.7 1996/06/12 15:39:29 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import koala.util.*;
import koala.look.*;
import java.awt.Component;
import java.awt.Container;
import java.awt.Event;
import java.awt.LayoutManager;
import java.awt.Dimension;
import java.awt.Insets;
import java.util.Observer;
import java.util.Observable;



public class Scrollbar extends Composite {
    public static final int HORIZONTAL = Thumb.HORIZONTAL;
    public static final int VERTICAL =  Thumb.VERTICAL;
    static final String UP = App.getStringProp("Scrollbar.upArrowFile",
						"arrowup.gif");
    static final String DOWN = App.getStringProp("Scrollbar.downArrowFile",
						"arrowdown.gif");
    static final String LEFT = App.getStringProp("Scrollbar.leftArrowFile",
						"arrowleft.gif");
    static final String RIGHT = App.getStringProp("Scrollbar.rightArrowFile",
						"arrowright.gif");

    protected Button down, up;
    protected Slider thumb;
    protected Adjustment adjust;
    
    public Scrollbar(int orient, Scrollee s) {
	initScrollbar(orient, s, new ActiveAdjustment());
    }
    
    public Scrollbar(Scrollee s) {
	initScrollbar(VERTICAL, s, new ActiveAdjustment());
    }
    
    public Scrollbar(Scrollee s, Adjustment a) {
	initScrollbar(VERTICAL, s, a);
    }
    
    public Scrollbar(int orient, Scrollee s, Adjustment a) {
	initScrollbar(orient, s, a);
    }
    
    protected void initScrollbar(int orient, Scrollee s, Adjustment a) {
	adjust = a;
	setLook(new BorderLook());
	setLayout(new ScrollbarLayout(orient));
	thumb = new Thumb(a, orient);
	add("Thumb", (Component)thumb);
	up = new Button(new Icon(this, (orient == VERTICAL) ? UP : LEFT), 
			new ScrollCmd(ScrollCmd.BACKWARD, s, a));
	down = new Button(new Icon(this, (orient == VERTICAL) ? DOWN : RIGHT),
			  new ScrollCmd(ScrollCmd.FORWARD, s, a));
	add("Up", up);
	add("Down", down);
	down.setBehavior(new SpinBehavior(down));
	up.setBehavior(new SpinBehavior(up));
    }
}
