/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: ScrollbarLayout.java,v 1.2 1996/06/12 15:39:30 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import koala.util.*;
import koala.look.*;
import java.awt.Component;
import java.awt.Container;
import java.awt.Event;
import java.awt.LayoutManager;
import java.awt.Dimension;
import java.awt.Insets;


class ScrollbarLayout implements LayoutManager {
    
    Component thumb, up, down;
    int orient = Scrollbar.VERTICAL;
    
    public ScrollbarLayout(int o) {
	if((o != Scrollbar.VERTICAL) && (o != Scrollbar.HORIZONTAL)) {
	    throw new IllegalArgumentException("invalid orientation");
	}
	orient = o;
    }

    /**
     * Adds the specified named component to the layout.
     * @param name the String name
     * @param comp the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {
	if(name.equals("Thumb")) {
	    thumb = comp;
	}
	else if(name.equals("Up")) {
	    up = comp;
	}
	else if(name.equals("Down")) {
	    down = comp;
	}
    }

    /**
     * Removes the specified component from the layout.
     * @param comp the component to be removed
     */
    public void removeLayoutComponent(Component comp) {
	if(comp == thumb) {
	    thumb = null;
	}
	else if(comp == up) {
	    up = null;
	}
	else if(comp == down) {
	    down = null;
	}
    }

    
    public Dimension minimumLayoutSize(Container target) {
	return preferredLayoutSize(target);
    }
     
    public Dimension verticalPreferredLayoutSize(Container target) {
	Dimension dim = new Dimension(0, 0);
	
	if ((thumb != null) && thumb.isVisible()) {
	    Dimension d = thumb.preferredSize();
	    dim.height += d.height;
	    dim.width = Math.max(d.width, dim.width);
	}
	
	if ((up != null) && up.isVisible()) {
	    Dimension d = up.preferredSize();
	    dim.height += d.height;
	    dim.width = Math.max(d.width, dim.width);
	}
	
	if ((down != null) && down.isVisible()) {
	    Dimension d = down.preferredSize();
	    dim.height += d.height;
	    dim.width = Math.max(d.width, dim.width);
	}

	return dim;
    }
     
    public Dimension horizontalPreferredLayoutSize(Container target) {
	Dimension dim = new Dimension(0, 0);
	
	if ((thumb != null) && thumb.isVisible()) {
	    Dimension d = thumb.preferredSize();
	    dim.width += d.width;
	    dim.height = Math.max(d.height, dim.height);
	}
	
	if ((up != null) && up.isVisible()) {
	    Dimension d = up.preferredSize();
	    dim.width += d.width;
	    dim.height = Math.max(d.height, dim.height);
	}
	
	if ((down != null) && down.isVisible()) {
	    Dimension d = down.preferredSize();
	    dim.width += d.width;
	    dim.height = Math.max(d.height, dim.height);
	}

	return dim;
    }
     
    public Dimension preferredLayoutSize(Container target) {
	switch (orient) {
	case Scrollbar.VERTICAL: return verticalPreferredLayoutSize(target);
	default: return horizontalPreferredLayoutSize(target);
	}
    }
    
    
    /**
     * Lays out the specified container.
     * @param target the component being laid out
     * @see Container
     */
    public void verticalLayoutContainer(Container target) {
	Insets insets = target.insets();
	int top = insets.top, left = insets.left;
	int width = target.size().width - left - insets.right;
	Dimension d;
	
	if(up != null) {
	    d = up.preferredSize();
	    up.reshape(left, top, width, d.height);
	    top += d.height;
	}
	if(down != null) {
	    d = down.preferredSize();
	    down.reshape(left, top, width, d.height);
	    top += d.height;
	}
	if(thumb != null) {
	    int height = target.size().height - top - insets.bottom;
	    thumb.reshape(left, top, width, height);
	}
    }
    
    
    /**
     * Lays out the specified container.
     * @param target the component being laid out
     * @see Container
     */
    protected void horizontalLayoutContainer(Container target) {
	Insets insets = target.insets();
	int top = insets.top, left = insets.left;
	int height = target.size().height - top - insets.bottom;
	Dimension d;
	
	if(up != null) {
	    d = up.preferredSize();
	    up.reshape(left, top, d.width, height);
	    left += d.width;
	    }
	if(down != null) {
	    d = down.preferredSize();
	    down.reshape(left, top, d.width, height);
	    left += d.width;
	}
	if(thumb != null) {
	    int width = target.size().width - left - insets.right;
	    thumb.reshape(left, top, width, height);
	}
    }
    
    /**
     * Lays out the specified container.
     * @param target the component being laid out
     * @see Container
     */
    public void layoutContainer(Container target) {
	switch(orient) {
	case Scrollbar.VERTICAL:
	    verticalLayoutContainer(target); break;
	default:
	    horizontalLayoutContainer(target);
	}
    }
}
