/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: SpinBehavior.java,v 1.3 1996/06/05 15:34:56 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import java.awt.Component;
import java.awt.Event;

public class SpinBehavior implements Behavior, Runnable {
    static private int count = 0;
    static protected int initialDelay = 
      Integer.getInteger("SpinBehavior.initialDelay", 300).intValue(); // 300ms
    static protected int repeatDelay = 
      Integer.getInteger("SpinBehavior.repeatDelay", 50).intValue(); // 50ms
    private ButtonWidget target;
    private Component last;
    private boolean pressed = false; // local to the behavior
    private Thread ticker = null;
    
    public SpinBehavior(ButtonWidget bc) {
	target = bc;
    }
    
    private Thread ticker() {
	if (ticker == null) {
	    ticker = new Thread(this, "SpinBehaviorThread # " + count);
	    count++;
	}
	return ticker;
    }

    public boolean handleEvent(Component c, Event evt) {
	last = c;
	switch (evt.id) {
            case Event.MOUSE_DOWN: {
		target.press((Widget)c, evt);
		pressed = true;
		ticker().start();
		return true;
            }
            case Event.MOUSE_ENTER: {
		if(pressed) {
		   target.press((Widget)c, evt);
		   ticker().resume();
	       }
		return true;
            }
            case Event.MOUSE_EXIT: {
		if(target.isPressed()) {
		   target.release((Widget)c, evt);
		   ticker().suspend();
	       }
		return true;
            }
	    case Event.MOUSE_UP: {
		if(target.isPressed()) {
		    target.release((Widget)c, evt);
		    target.activate((Widget)c, evt);
		}
		ticker().stop();
		ticker = null;
		pressed = false;
		return true;
	    }
	}
        return false;
    }
    
    public void run() {
	Thread me = Thread.currentThread();
	try {
	    me.sleep(initialDelay);
	    target.activate(target, null);
	    while(true) {
		me.sleep(repeatDelay);
		target.activate(target, null);
	    }
	}
	catch(InterruptedException e) {
	    System.out.println("Thread interrupted");
	}
    }
}
