/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: TextField.java,v 1.4 1996/06/18 08:31:33 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import koala.util.*;
import koala.look.*;

import java.awt.Event;
import java.awt.Component;
import java.awt.Graphics;
import java.util.Observer;
import java.util.Observable;
import java.awt.Dimension;
import java.awt.FontMetrics;

public class TextField extends Control implements Observer, Look, Behavior, LabelWidget {
    
    static protected Style style = new Style("TextField");
    ActiveString variable;
    InsertionPoint ip;
    
    public TextField() {
	this(new ActiveString());
    }
    
    public TextField(ActiveString v) {
	variable = v;
	ip = variable.insertionPoint();
	variable.addObserver(this);
	setStyle(style);
	setLook(new BorderLook(this));
	setBehavior(this);
    }

    public ActiveString variable() {
	return variable;
    }

    public boolean handleEvent(Component c, Event evt) {
	if(evt.id == Event.KEY_PRESS) {
	    if(evt.controlDown()) {
		switch(evt.key) {
		case 1:
		    ip.beginningOfLine(); repaint(); break;
		case 2:
		    ip.backwardChar(); repaint(); break;
		case 6:
		    ip.forwardChar(); repaint(); break;
		case 5:
		    ip.endOfLine(); repaint(); break;
		case 4:
		    variable.deleteForwardChar(ip); break;
		case 8:
		    variable.deleteBackwardChar(ip); break;
		}
	    }
	    else {
		switch(evt.key) {
		case 8:
		    variable.deleteBackwardChar(ip); break;
		case 127:
		    variable.deleteForwardChar(ip); break;
		default:
		    variable.insertChar((char)evt.key, ip);
		}
	    }
	    return true;
	}
	else if(evt.id == Event.KEY_ACTION) {
	    switch(evt.key) {
	    case Event.END:
		ip.endOfLine(); break;
	    case Event.HOME:
		ip.beginningOfLine(); break;
	    case Event.LEFT:
		ip.backwardChar(); break;
	    case Event.RIGHT:
		ip.forwardChar(); break;
	    }
	    repaint();
	    return true;
	}
	return false;
    }
 
    public void update(Observable o, Object arg) {
	repaint();
    }

    //============================================================
    //      implements: LabelWidget
    //============================================================

    public void setLabel(String s) {
	variable.set(s);
    }
    public String getLabel() {
	return variable.get();
    }
    // inherited from Component
    // public boolean isEnabled();
	

    //============================================================
    //      implements: Look
    //============================================================

    Dimension size;
    public void draw(DrawingContext dc) {
	ip.check();
	int h;
	String s = variable.get();
	Graphics g = dc.graphics();
	Dimension clip = dc.size();
	FontMetrics fm = g.getFontMetrics();
	
 	h = fm.getHeight();
 	dc.drawString(s, 0,
 		     (clip.height-h)/2+fm.getAscent());
	
	// compute cursor position
	int w = fm.stringWidth(variable.left(ip));
	dc.drawLine(w-1, 2, w-1, clip.height-4);
	dc.drawLine(w, 2, w, clip.height-4);
    }
    
    public Dimension request(DrawingContext dc) {
	Graphics g = dc.graphics();
	String s = variable.get();

	FontMetrics fm = g.getFontMetrics();
	size =  new Dimension(fm.stringWidth(s), 
			      fm.getHeight()+6);
	return size;
    }
    
    public Dimension size(DrawingContext dc) {
	return (size != null) ? size : request(dc);
    }
}
