/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Thumb.java,v 1.8 1996/06/12 15:39:30 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.gui.*;
import koala.util.*;
import koala.look.*;
import java.awt.Component;
import java.awt.Container;
import java.awt.Event;
import java.awt.Dimension;
import java.util.Observer;
import java.util.Observable;

public class Thumb extends Control implements Slider, Observer, Look, Scrollee {
    static protected Style style = new Style("Thumb");
    public static final int HORIZONTAL = 1;
    public static final int VERTICAL = 2;
    protected static int SLIDERWIDTH =  
      Integer.getInteger("Thumb.sliderWidth", 14).intValue();
    
    protected Scrollee target;
    protected Adjustment variable;
    protected int orientation;
    protected boolean dragging = false;

    public Thumb(Scrollee s, Adjustment a, int orient) {
	target = s;
	variable = a;
	orientation = orient;
	setLook(this);
	setStyle(style);
	setBehavior(new DragBehavior(this));
    }

    public Thumb(Scrollee s, Adjustment a) {
	init(s, a, HORIZONTAL);
    }

    public Thumb(Adjustment a) {
	init(this, a, HORIZONTAL);
    }

    public Thumb(Adjustment a, int orient) {
	init(this, a, orient);
    }

    private void init(Scrollee s, Adjustment a, int orient) {
	target = s;
	variable = a;
	variable.addObserver(this);
	orientation = orient;
	setLook(this);
	setStyle(style);
	setBehavior(new DragBehavior(this));
    }

    public int major() {
	return (orientation == HORIZONTAL) ? size().width : size().height;
    }

    public int minor() {
	return (orientation == HORIZONTAL) ? size().height : size().width;
    }

    public int sliderSize() {
	return (int) (major() * variable.fraction() );
    }

    public int sliderWidth() {
	return SLIDERWIDTH;
    }

    public int sliderPos() {
	return (int)((major() -sliderSize()) * variable.value());
    }

    //============================================================
    //      implements: Slider
    //============================================================

    public Adjustment variable() {
	return variable;
    }

    public void select(int x, int y, Event evt) {
	int pos = (orientation == HORIZONTAL) ? x : y;
	int ssize = sliderSize();
	int spos = sliderPos();
	if(pos < spos) {
	    target.pageBackward(variable);
	}
	else if(pos > spos+ssize) {
	    target.pageForward(variable);
	}
	else {
	    dragging = true;
	}
    }
    
    public void drag(int dx, int dy, Event evt) {
	if(dragging) {
	    float dv = (float)((orientation == HORIZONTAL) ? dx : dy);
	    float delta = (float)dv / (float)(major()-sliderSize());
	    target.jumpTo(variable, variable.value()+delta);
	}
    }
    
    public void drop(int x, int y, Event evt) {
	    dragging = false;
    }

    public void jumpto(int x, int y, Event evt) {
	
    }

    public void enter(int x, int y, Event evt) {
	// nothing
    }

    public void leave(int x, int y, Event evt) {
	// nothing
    }
    
    //============================================================
    //      implements: Observer
    //============================================================

    public void update(Observable o, Object v) {
	repaint();
    }
    
    //============================================================
    //      implements: Look
    //============================================================
    
    public void draw(DrawingContext dc) {
	Dimension clip = dc.size();
	int ssize = sliderSize()-1;
	int spos = sliderPos();
	if(orientation == HORIZONTAL) {
  	    dc.fillRelief(spos, 0, ssize, 
  			  clip.height-1, Relief.RAISED,
			  dc.style().shadowThickness);
	}
	else {
 	    dc.fillRelief(0, spos,  
 			  clip.width-1, ssize, Relief.RAISED, 
			  dc.style().shadowThickness);
	}
    }
    
    public Dimension request(DrawingContext dc) {
	if(orientation == HORIZONTAL) {
	    return new Dimension(200, sliderWidth());
	}
	else {
	    return new Dimension(sliderWidth(), 200);
	}
    }
    
    public Dimension size(DrawingContext dc) {
	return request(dc);
    }
    

    //============================================================
    //      implements: Scrollee
    //============================================================


    public void scrollForward(Adjustment ref) {
	variable.translate(0.01f, 0.0f);
    }
    public void scrollBackward(Adjustment ref) {
	variable.translate(-0.01f, 0.0f);
    }
    public void pageForward(Adjustment ref) {
	variable.translate(0.1f, 0.0f);
    }
    public void pageBackward(Adjustment ref) {
	variable.translate(-0.1f, 0.0f);
    }
    public void jumpTo(Adjustment ref, float v) {
	variable.set(v, variable.fraction());
    }
}
