/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: VerticalLayout.java,v 1.3 1996/05/29 16:35:23 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.util.*;

import java.awt.*;

/**
 * A simple layout that arranges its components vertically
 * all the components are given the same width and keep their own height.
 * 
 */

public class VerticalLayout implements LayoutManager {

    final static private int VGAP = 
      Integer.getInteger("VerticalLayout.vgap", 0).intValue();
    final static private int HMARGIN =
      Integer.getInteger("VerticalLayout.hmargin", 0).intValue();

    private int vgap;
    private int hmargin;
    
    /**
     * Constructs a new VerticalLayout.
     */
    public VerticalLayout() {
	this(VGAP, HMARGIN);
    }
    
    /**
     * Constructs a new VerticalLayout with specific gap and margin.
     */
    public VerticalLayout(int gap, int margin) {
	vgap = gap;
	hmargin = margin;
    }

    /**
     * Adds the specified named component to the layout.
     * @param name the String name
     * @param comp the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {}

    /**
     * Removes the specified component from the layout.
     * @param comp the component to be removed
     */
    public void removeLayoutComponent(Component comp) {}

    
    public Dimension minimumLayoutSize(Container target) {
	return preferredLayoutSize(target);
    }
     
    public Dimension preferredLayoutSize(Container target) {
	Dimension dim = new Dimension(0, 0);
	int width = 0;
	int height = 0;
	Component children[] = target.getComponents();
	
	for(int i=0; i < children.length; i++) {
	    width = Math.max(width, children[i].preferredSize().width);
	    height += children[i].preferredSize().height;
	}
	height += vgap * (children.length + 1);
	Insets insets = target.insets();
	dim.width = width + insets.left + insets.right + hmargin*2;
	dim.height = height + insets.top + insets.bottom;

	return dim;
    }
    
    
    /**
     * Lays out the specified container.
     * @param target the component being laid out
     * @see Container
     */
    public void layoutContainer(Container target) {
	Insets insets = target.insets();
	int top = insets.top, left = insets.left + hmargin;
	int width = target.size().width - left - insets.right - hmargin;
	Component children[] = target.getComponents();
	
	top += vgap;
	for(int i=0; i < children.length; i++) {
	    int h = children[i].preferredSize().height;
	    children[i].reshape(left, top, width, h);
	    top += h+vgap;
	}
    }
     
    /**
     * Returns the String representation of this BorderLayout's values.
     */
    public String toString() {
	return getClass().getName() + ",vgap=" + vgap + "]";
    }

}
