/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: BorderLook.java,v 1.6 1996/06/05 15:34:57 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */


package koala.look;

import koala.gui.*;
import koala.util.*;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Container;
import java.awt.Dimension;

public class BorderLook implements Look {
    
    protected Look body;
    Dimension size = null;

    public BorderLook(Look bod) {
	init(bod);
    }

    public BorderLook() {
	this(null);
    }
    
    private void init(Look bod) {
	body = bod;
    }
    
    public void draw(DrawingContext dc) {
	int w, h, thickness, bevel;
	Style s = dc.style();
	bevel = s.relief;
	thickness = s.shadowThickness;
	w = dc.size().width - 1;
	h = dc.size().height - 1;
	dc.setColor(s.background);
	dc.drawRelief(0, 0, w, h, bevel, thickness);
	dc.setColor(s.foreground);
	
	if(body != null) {
	    Dimension dim = dc.size();
	    DrawingContext bodydc = new DrawingContext(dc);
	    bodydc.resize(dim.width - thickness*2, 
			  dim.height - thickness*2);
	    bodydc.translate(thickness, thickness);
	    body.draw(bodydc);
	}
    }
    
    public Dimension request(DrawingContext dc) {
	int thickness = dc.style().shadowThickness * 2;
	size = new Dimension(thickness, thickness);
	if(body != null) {
	    Dimension r2 = body.request(dc);
	    size.width += r2.width;
	    size.height += r2.height;
	}
	App.debug("BorderLook requesting " + size);
	return size;
    }
    
    public Dimension size(DrawingContext dc) {
	return (size != null) ? size : request(dc);
    }
}
