/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Icon.java,v 1.4 1996/06/05 15:34:58 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.look;

import koala.gui.*;
import koala.util.*;
import java.awt.image.ImageObserver;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.Dimension;
import java.awt.MediaTracker;
import java.awt.Component;

public class Icon implements Look {
    static final int defsize = 32;
    String iname = null;
    Image img = null;
    ImageObserver obs = null;
    Dimension size = null;
   
    public Icon(Component c, String name) {
	iname = "<" + name + ">";
	try {
	    img = Toolkit.getDefaultToolkit().getImage(locateFile(name));
	}
	catch(SecurityException e) {
	    System.out.println("could not load image " + name);
	    return;
	}
	obs = c;
        MediaTracker tracker = new MediaTracker(c);
        tracker.addImage(img, 0);
        
        try {
            tracker.waitForID(0);
        } 
        catch (InterruptedException e) {
            img = null;
            System.out.println("Could not load image: " + name);
            return;
        }
        // this is the only way I found to see if the image is loaded
        if(img.getWidth(obs) == -1) {
            img = null;
        }
	else {
	    size = new Dimension(img.getWidth(obs), img.getHeight(obs));
	}

        if (img == null)  {
            System.out.println("Could not load image: " + name);
        }
    }
    
    protected String locateFile(String name) {
	String path = App.getStringProp(Gui.IMAGE_PATH_P, ".");
	return path + "/" + name;
    }
    
    public void draw(DrawingContext dc) {
	App.debug("Icon drawing into " + dc.size());
        if (img != null) {
            dc.drawImage(img, 0, 0, obs);
        }
        else {
            dc.drawString(iname, 0, 20);
        }
    }
    
    public Dimension request(DrawingContext dc) {
        if(img != null) {
            return size;
        }
	else {
	    return new Dimension(defsize, defsize);
	}
    }
    
    public Dimension size(DrawingContext dc) {
	return (size != null) ? size : request(dc);
    }
}
