/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: SeparatorLook.java,v 1.1 1996/06/18 08:31:33 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.look;

import koala.gui.*;
import koala.util.*;
import koala.kit.Separator;	// for consts

import java.awt.Dimension;


/**
 * The Separator Look class.
 *
 * depending on its 'orientation' fields, this class draw a separator line,
 * vertically or horizontally.
 */
public class SeparatorLook implements Look {
    protected int orientation;


   /**
    * builds a new Separator look.
    *
    * @param orient the orientation for this look.
    */    
    public SeparatorLook(int orient) {
	orientation = orient;
    }

   /**
    * builds a new Separator look.
    *
    * orientation is set to HORIZONTAL.
    */   
    public SeparatorLook() {
	this(Separator.HORIZONTAL);
    }

   /**
    * Draws a vertical or horizontal relief line. If orientation is HORIZONTAL,
    * the line is vertically centered. If orientation is VERTICAL, it is
    * horizontally centered.
    */
    public void draw(DrawingContext dc) {
	Dimension clip = dc.size();
	Style s = dc.style();
	dc.setColor(s.background);
	if(orientation == Separator.HORIZONTAL) {
	    dc.drawRelief(0, clip.height/2-1, clip.width, 1,
			  s.relief, s.shadowThickness);
	}
	else {
	    
	    dc.drawRelief(clip.width/2-1, 0, 1, clip.height,
			  s.relief, s.shadowThickness);
	}
	dc.setColor(s.foreground);
    }

   /**
    * Compute size.
    */    
    public Dimension request(DrawingContext dc) {
	if(orientation == Separator.HORIZONTAL) {
	    return new Dimension(100, 2);
	}
	else {
	    return new Dimension(2, 100);
	}
    }

   /**
    * Return size.
    */  
    public Dimension size(DrawingContext dc) {
	return request(dc);
    }
}
