/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: HBoxLayout.java,v 1.1 1996/06/13 14:30:39 leon Exp leon $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.misc;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.Insets;

public class HBoxLayout extends BoxLayout {

    public HBoxLayout() {
	super();
    }

    public HBoxLayout(int gap, int maj, int min) {
	super(gap, maj, min);
    }
    
    public Dimension preferredLayoutSize(Container target) {
	Dimension dim = new Dimension(0, 0);
	int width = 0;
	int height = 0;
	Component children[] = target.getComponents();
	
	for(int i=0; i < children.length; i++) {
	    height = Math.max(height, children[i].preferredSize().height);
	    width += children[i].preferredSize().width;
	}
	width += gap * (children.length + 1) + majorm*2*children.length;
	Insets insets = target.insets();
	dim.width = width + insets.left + insets.right;
	dim.height = height + insets.top + insets.bottom + minorm*2;

	return dim;
    }

    public void layoutContainer(Container target) {
	Dimension size = target.size();
	Insets insets = target.insets();
	int top = insets.top + minorm, left = insets.left;
	int height = size.height - top - insets.bottom - minorm;
	Component children[] = target.getComponents();
	Dimension boxes[] = new Dimension[children.length];
	
	int cwidth = 0;		// total width of children
	int hspace = 0;
	for(int i = 0; i < children.length; i++) {
	    boxes[i] = children[i].preferredSize();
	    cwidth += boxes[i].width;
	    BoxLayoutHint blh = hints.get(children[i]);
	    if(blh != null) {
		hspace += blh.left + blh.right + blh.width;
	    }
	}
	int extraHspace = size.width - insets.left - insets.right - cwidth;
	for(int i=0; i < children.length; i++) {
	    int w = boxes[i].width + majorm*2;
	    int dh = 0;
	    int right = 0, dy = 0;
	    BoxLayoutHint blh = hints.get(children[i]);
	    if(blh != null) {
		int vspace = blh.top + blh.bottom + blh.height;
		int extraVspace = height - boxes[i].height;
		if(vspace != 0) {
		    dh = (blh.height * extraVspace) / vspace;
		    dy = (blh.top  * extraVspace) / vspace;
		}
		if(hspace != 0) { 
		    w += (blh.width * extraHspace) / hspace;
		    left += (blh.left * extraHspace) / hspace;
		    right = (blh.right * extraHspace) / hspace;
		}
	    }
	    children[i].reshape(left, top+dy, w, boxes[i].height+dh);
	    left += w + gap + right;
	}
    }
}
