/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: VBoxLayout.java,v 1.1 1996/06/13 15:23:31 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.misc;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.Insets;

public class VBoxLayout extends BoxLayout {
     
    public Dimension preferredLayoutSize(Container target) {
	Dimension dim = new Dimension(0, 0);
	int width = 0;
	int height = 0;
	Component children[] = target.getComponents();
	
	for(int i=0; i < children.length; i++) {
	    Dimension csize = children[i].preferredSize();
	    width = Math.max(width, csize.width);
	    height += csize.height;
	}
	height += gap * (children.length + 1) + majorm*2*children.length;
	Insets insets = target.insets();
	dim.width = width + insets.left + insets.right + minorm*2;
	dim.height = height + insets.top + insets.bottom;

	return dim;
    }
    

    public void layoutContainer(Container target) {
	Dimension size = target.size();
	Insets insets = target.insets();
	int left = insets.left + minorm;
	int top = insets.top;
	int width = size.width  - left - insets.right - minorm;
	Component children[] = target.getComponents();
	Dimension boxes[] = new Dimension[children.length];
	
	int cheight = 0;		// total height of children
	int vspace = 0;
	for(int i = 0; i < children.length; i++) {
	    boxes[i] = children[i].preferredSize();
	    cheight += boxes[i].height;
	    BoxLayoutHint blh = hints.get(children[i]);
	    if(blh != null) {
		vspace += blh.top + blh.bottom + blh.height;
	    }
	}
	int extraVspace = size.height - insets.top - insets.bottom - cheight;
	for(int i=0; i < children.length; i++) {
	    int h = boxes[i].height + majorm*2;
	    int dh = 0;
	    int bottom = 0, dx = 0;
	    BoxLayoutHint blh = hints.get(children[i]);
	    if(blh != null) {
		int hspace = blh.left + blh.right + blh.width;
		int extraHspace = width - boxes[i].width;
		if(hspace != 0) {
		    dh = (blh.width * extraHspace) / hspace;
		    dx = (blh.left  * extraHspace) / hspace;
		}
		if(vspace != 0) { 
		    h += (blh.height * extraVspace) / vspace;
		    top += (blh.top * extraVspace) / vspace;
		    bottom = (blh.bottom * extraVspace) / vspace;
		}
	    }
	    children[i].reshape(left+dx, top, boxes[i].width+dh, h);
	    top += h + gap + bottom;
	}
    }
}
