/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: ActiveNumber.java,v 1.4 1996/06/18 16:12:21 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.util;

/**
 * An active implementation of float that notifies its changes to its
 * observers.
 */
public class ActiveNumber extends Subject implements NumericModel {
    protected float value = 0;
    
   /**
    * Sets a new float value. Triggers the update method for attached
    * observers.
    */
    public void set(float v) {
	value = v;
	setChanged();
	notifyObservers();
    }
    
   /**
    * Returns the current value.
    */
    public float get() {
	return value;
    }

   /**
    * Increments value. Triggers the update method for attached observers.
    */    
    public void inc() {
	value++;
	setChanged();
	notifyObservers();
    }

   /**
    * Decrements value. Triggers the update method for attached observers
    */    
    public void dec() {
	value--;
	setChanged();
	notifyObservers();
    }
}
