/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: ActiveSequence.java,v 1.2 1996/06/18 16:12:22 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.util;

import java.util.*;

/**
 * An Active implementation of Sequence that notifies its changes to its
 * observers
 */
public class ActiveSequence extends Subject implements SequenceModel {

    protected Vector v = null;

   /**
    * Builds a new empty sequence.
    */
    public ActiveSequence () {
	v = new Vector();
    }

    //============================================================
    //      implements: SequenceModel
    //============================================================

    private final void commit() {
	setChanged();
	notifyObservers();
    }

   /**
    * Inserts an item at position 'n'. Triggers the update method for attached
    * observers.
    */
    public void insert(int n, Object item) {
	v.insertElementAt(item, n);
	commit();
    }

   /**
    * Appends an item. Triggers the update method for attached observers.
    */
    public void insert(Object item) {
	v.addElement(item);
	commit();
    }

   /**
    * Removes an item. Triggers the update method for attached observers.
    */
    public void remove(Object item) {
	v.removeElement(item);
	commit();
    }

   /**
    * Removes item at position 'n'. Triggers the update method for attached
    * observers.
    */
    public void remove(int n) {
	v.removeElementAt(n);
	commit();
    }

   /**
    * Removes all elements. Triggers the update method for attached observers.
    */    
    public void removeAll() {
	v.removeAllElements();
	commit();
    }

   /**
    * Returns element at position 'n'.
    */    
    public Object elementAt(int n) {
	return v.elementAt(n);
    }

   /**
    * Returns an Enumeration of elements.
    */   
    public Enumeration elements() {
	return v.elements();
    }

   /**
    * Returns true if the elements is contained, false otherwise.
    */
    public boolean contains(Object item) {
	return v.contains(item);
    }

   /**
    * Returns the number of elements.
    */    
    public int size() {
	return v.size();
    }
    
}
