/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: ActiveString.java,v 1.4 1996/06/18 16:12:22 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.util;

import koala.gui.*;

import java.util.Observer;

class SimpleInsertionPoint implements InsertionPoint {

    int pos = 0;
    ActiveString variable;

    public SimpleInsertionPoint(ActiveString a) {
	variable = a;
    }

    //============================================================
    //      implements: InsertionPoint
    //============================================================

    public void forwardChar(){
	pos++;
	if(pos > variable.contents.length()) {
	    pos = variable.contents.length();
	}
    }

    public void check() {
	if(pos < 0) pos = 0;
	else if(pos > variable.contents.length()) {
	    pos = variable.contents.length();
	}
    }
    
    public void backwardChar() {
	pos--;
	if(pos < 0) pos = 0;
    }

    public void endOfLine() {
	pos = variable.contents.length();
    }
    
    public void beginningOfLine() {
	pos = 0;
    }
    
    public int pos() {
	return pos;
    }
}


/**
 * An active implementation of String that supports TextInput and Observers.
 */
public class ActiveString extends Subject implements StringModel,TextInput {

    MutableString contents;

   /**
    * Builds a new empty ActiveString.
    */    
    public ActiveString() {
	contents = new MutableString("");
    }

   /**
    * Creates and returns a new InsertionPoint object.
    */    
    public InsertionPoint insertionPoint() {
	return new SimpleInsertionPoint(this);
    }
    
    //============================================================
    //      implements: StringModel
    //============================================================

   /**
    * Sets the model's value. This notifies changes to the observers.
    * @param v the new string value.
    */    
    public void set(String v) {
	contents = new MutableString(v);
	setChanged();
	notifyObservers();
    }

   /**
    * Returns the model's value.
    */    
    public String get() {
	return contents.string();
    }
    
    //============================================================
    //      implements: TextInput
    //============================================================

   /**
    * Inserts a character at the given insertion point. notifies to attached
    * observers.
    */
    public void insertChar(char c, InsertionPoint ip) {
	contents.insert(ip.pos(), c);
	ip.forwardChar();
	setChanged();
	notifyObservers();
    }

   /**
    * Deletes backward char, regarding to insertion point. Notifies to attached
    * observers.
    */
    public void deleteBackwardChar(InsertionPoint ip) {
	if(ip.pos() > 0) {
	    contents.delete(ip.pos());
	    ip.backwardChar();
	    setChanged();
	    notifyObservers();
	}
    }

   /**
    * Deletes foreward char, regarding to insertion point. Notifies to attached
    * observers.
    */
    public void deleteForwardChar(InsertionPoint ip) {
	if(ip.pos() < contents.length()) {
	    contents.delete(ip.pos());
	    setChanged();
	    notifyObservers();
	}
    }

   /**
    * Returns the sub string, on left side of the insertion point.
    */    
    public String left(InsertionPoint ip) {
	if(contents.length() != 0) {
	    int pos = ip.pos();
	    return contents.string(0, pos);
	}
	else {
	    return "";
	}
    }
}
