/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Adjustment.java,v 1.2 1996/06/18 16:12:23 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.util;


/**
 * A Adjustment is a couple of float values, between two bounds inclusive, used
 * to represent for an amount and a percent of something. Typically,
 * Adjustments are used to represent the value position (position) and fraction
 * (perCent) of a scrollbar.
 */
public interface Adjustment extends Model {
   /**
    * Sets new values to the Adjustment.
    *
    * @param v The new value.
    * @param f The new fraction.
    */
    void set(float v, float f);
    
   /**
    * Adds the specified values as value and fraction.
    *
    * @param dv The delta added to the value: final value is constrained to
    * stay between the bounds
    *
    * @param df The delta added to the fraction: final fraction is constrained
    * to stay between (high bound - low bound) */
    void translate(float dv, float df);

   /**
    * @return the current value.
    */   
    float value();

   /**
    * @return  the current fraction.
    */    
    float fraction();
}
