/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Model.java,v 1.3 1996/06/18 13:56:39 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.util;

import java.util.Observer;

/**
 * The Model interface.
 * 
 * This is the definition of the MVC's Model: A Model is an object holding a
 * value and able to emit messages when the value is modified. Objects
 * receiving notification messages are called Observers.  A model may be
 * disabled/enabled. disabling a model may be useful to perform more than one
 * modification but update observers only once. An implementation of Model is
 * given by the Subject class.
 *
 * @see Subject
 */
interface Model {

   /**
    * Attaches an observer to the model.
    *
    * @param o the observer to be attached.  */
    public void addObserver(Observer o);

   /**
    * Detaches an observer to the model.
    *
    * @param o the observer to be detached.
    */    
    public void deleteObserver(Observer o);

   /**
    * Notify changes to attached observers.
    *
    * if the model has been modified and if the model is enabled, this calls
    * the update() method for any attached observer.
    */
    public void notifyObservers();
    
   /**
    * Detaches any attached observer.
    */    
    public void deleteObservers();

   /**
    * returns the number of observers curretly attached.
    */    
    public int countObservers();

   /**
    * disables update notification.
    */    
    public void disable();

   /**
    * enables update notification. It calls notifyObservers, so that is the
    * model was modified it updates observers now.
    */
    public void enable();
}
