/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: ScrollCmd.java,v 1.4 1996/06/18 16:12:25 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.util;

import koala.gui.*;

import java.awt.Event;
import java.awt.Component;

/**
 * A ScrollCmd sends scrolling orders to a Scrollee, along a specified
 * referree.
 */
public class ScrollCmd implements Command {
    static public final int FORWARD = 1;
    static public final int BACKWARD = 2;
    static public final int PAGE_FORWARD = 3;
    static public final int PAGE_BACKWARD = 4;

    Scrollee variable;
    Adjustment referee;
    int scroll;

   /**
    * Builds a new ScrollCmd.
    *
    * @param cmd The scrolling command to be used (FORWARD, BACKWARD,
    * PAGE_FORWARD, PAGE_BACKWARD
    * @param targ the target Scrollee.
    * @param ref The scrolling referee
    */
    public ScrollCmd(int cmd, Scrollee targ, Adjustment ref) {
	if((cmd < FORWARD) || (cmd > PAGE_BACKWARD)) {
	    throw new IllegalArgumentException("cmd = " + cmd);
	}
	scroll = cmd;
	variable = targ;
	referee = ref;
    }

   /**
    * Depending
    */    
    public void execute(Component target, Event evt) {
	switch(scroll) {
	case FORWARD:
	    variable.scrollForward(referee);
	    break;
	case BACKWARD:
	    variable.scrollBackward(referee);
	    break;
	case PAGE_FORWARD:
	    variable.pageForward(referee);
	    break;
	case PAGE_BACKWARD:
	    variable.pageBackward(referee);
	    break;
	}
    }
}
